#!/usr/bin/env og
# ==
# --Project: Open Grid
# --Author: Sing.Yu <scaner@gmail.com>
# --Client-Script: List all nodes with meter info in the open-grid
# ==


import pprint
import og.lib.network

import grid_info
from numacomp import numacomp

def formatnodes(title, nodes, perline = 10):
	lines = []
	spaces = ' ' * (len(title))
	nodes = sorted(nodes, cmp = numacomp)
	first = True
	while nodes: 
		nodes0 = nodes[:perline]
		nodes = nodes[perline:]
		if first:
			lines.append('%s%s' % (title, ','.join(nodes0)))
			first = False
		else:
			lines.append('%s%s' % (spaces, ','.join(nodes0)))
		if nodes:
			lines[-1] += ','
	
	return '\n'.join(lines)
		
C = grid_info.OGFarmClient()

res = C.Call({
	'_': 'F.queryinfo',
	'grids': True,
	'nodes': False,
})['=']

total = {'node': 0, 'grid': 0, 'offline': 0}
for gname in sorted(res['grids']):
	nodeinfo = res['nodes'][gname]
	nodes = len(nodeinfo['register'])
	onlines = len(nodeinfo['online'])
	offlines = nodes - onlines

	total['grid'] += 1
	total['node'] += nodes

	onlinenodes = nodeinfo['online']
	offlinenodes = list(set(nodeinfo['register']) - set(nodeinfo['online']))
	unknownnodes = list(set(nodeinfo['online']) - set(nodeinfo['register']))
	total['offline'] += len(offlinenodes)

	print '--' 
	print 'Grid: %s, %d online/%d nodes' % (gname, onlines, nodes)
	if onlinenodes:
		print formatnodes('UP ', onlinenodes)
		print
	if offlinenodes:
		print formatnodes('DN ', offlinenodes)
		print
	if unknownnodes:
		print formatnodes('UN ', unknownnodes)
		print
	
print '=='
print 'Total %d grids, %d nodes, %d offline' % (
	total['grid'], total['node'], total['offline'])
	
		

C.close()
