#!/usr/bin/env og
# ==
# --Project: Open Grid
# --Author: Sing.Yu <scaner@gmail.com>
# --Client-Script: List all nodes with meter info in the open-grid
# ==


import pprint
import og.lib.network

import grid_info

from numacomp import numacomp

C =grid_info.OGClient()
meterinfo = C.Call({
	'_': 'S.nodemeterlist', 
	'!': {':': 'hive'},
	})['=']

for nid in sorted(meterinfo.keys(), cmp = numacomp):
	meter = meterinfo[nid]['meter']
	cpuinfo = meter['cpu']
	cpu = cpuinfo['modelname']
	if cpuinfo['ht'] != 1:
		cpu += ' HT'
	if cpuinfo['cpus'] != 1:
		cpu += ' x %d' % cpuinfo['cpus']
	
	meminfo = meter['mem']
	mem = '%.1fG' % (meminfo['MemTotal'] / 1024.0 / 1024)

	diskinfo = meter['disk']
	disk = []
	for mount, vfs in diskinfo.items():
		disk.append('%s=%dG/%dG' % (mount,
			int(vfs[0] * vfs[4] / 1024.0 / 1024.0 / 1024.0),
			int(vfs[0] * vfs[2] / 1024.0 / 1024.0 / 1024.0)))
	
	uptime =  int(float(meter['sys']['uptime'][0]))
	s = uptime % 60
	m = (uptime / 60) % 60
	h = (uptime / 3600) % 24
	d = (uptime / 86400)
	uptime = '%d days %02d:%02d:%02d' % (d, h, m, s)
	loadavg = ' '.join(meter['sys']['loadavg'][:3])

	print '= %s = load %s, uptime: %s' % (nid, loadavg, uptime)
	print '  CPU: %s, MEM:%s' % (cpu, mem)
	print '  DISK: %s' % (', '.join(disk))
	print '  --'

C.close()
