#!/usr/bin/python

# --
import os
import glob
import md5

BASEURL = 'http://ss-admin.io8.org/pypi'
BASE = '/var/www/ss-admin/html/pypi'

def LoadSign(abspath):
	signfn = abspath + '/.sign'
	sign = {}
	if not os.path.exists(signfn):
		return sign
	
	fin = file(signfn, 'r')
	for line in fin:
		fn, signstr = line.split()
		sign[fn] = signstr
	fin.close()

	return sign

def SaveSign(abspath, sign):
	if not sign: return

	signfn = abspath + '/.sign'
	fout = file(signfn, 'w')
	ks = sign.keys()
	ks.sort()
	for k in ks:
		fout.write('%s %s\n' % (k, sign[k]))
	fout.close()
	
def Sign(fname):
	m = md5.md5()
	fin = file(fname, 'r')
	m.update(fin.read())
	fin.close()
	return m.hexdigest()
	
def GenerateIndex(pname):
	print 'Generate Index for %s' % pname
	abspath = BASE + '/' + pname

	signcache = LoadSign(abspath)
	signchanged = False

	eggs = glob.glob(abspath + '/%s-*' % pname)

	for eggfn in eggs:
		bname = os.path.basename(eggfn)
		if bname in signcache:
			continue
		signcache[bname] = Sign(eggfn)
		signchanged = True
	
	if signchanged == True:
		SaveSign(abspath, signcache)

		fout = file(abspath + '/index.html', 'w')
		fout.write("<html><head><title>%s"
			"- PyPI Index</title></head><body><ul>\n" % pname)

		ks = signcache.keys()
		ks.sort()
		for k in ks:
			fout.write("""<li>
<a rel="download" href="%s/%s/%s#md5=%s">%s</a>
</li>
""" % (BASEURL, pname, k, signcache[k], k))
		fout.write("</ul></body><html>\n")
		fout.close()

def Main():
	# -- ScanDirs
	projects = glob.glob(BASE + '/*')

	for p in projects:
		if os.path.isdir(p):
			GenerateIndex(os.path.basename(p))

if __name__ == '__main__':
	Main()
