# ==
# --Project: Open Grid
# --Author: Sing.Yu <scaner@gmail.com>
# --Module: og.lib.logging
# ==

# --
# This is a logging module for OpenGrid.
# --

__all__ = ('_L', '_D', '_I', '_O', 'Current', 'UpdateCurrent')

import sys
import time
import socket

from og import constants

# ---
class LogCore:
	def __init__(self, nodeid = 'bootup'):
		self.nodeid = nodeid
		self.stdout = True
		self.udp = None

	# -- LogCore.setup
	def setup(self, C, sname = 'SRV'):
		self.setNodeId(C['nodeid'], sname)
		self.setUDPAddr(('ulog--' + C['boot-param']['domain'], constants.ULOG_PORT))
		#self.I('Use %s as boot config' % C['cfname'])

	# -- LogCore.setUDPAddr
	def setUDPAddr(self, addr):
		self.udp = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, 0)
		self.udp.connect(addr)

	# -- LogCore.setUDP
	def setUDP(self, s):
		self.udp = s
	
	def disableUDP(self):
		if self.udp:
			self.udp.close()
			self.udp = None

	def setNodeId(self, nodeid, sname = 'SRV'):
		self.nodeid = '%s:%s' % (nodeid, sname)

	# ---
	def logging(self, msg, tag = 'L'):
		if self.stdout:
			print '%s %c %s' % (
				time.strftime('%m%d\'%H%M%S'), tag, msg)
			sys.stdout.flush()
		if self.udp:
			try:
				self.udp.send('\0'.join((tag, self.nodeid, msg)))
			except socket.error:
				pass
				
	# ---
	def L(self, msg):
		self.logging(msg, 'L')
	
	def D(self, msg):
		self.logging(msg, 'D')
	
	def I(self, msg):
		self.logging(msg, 'I')

logCore = LogCore()
C = logCore

_C = C
_L = C.L
_I = C.I
_D = C.D
_O = C.logging

Logging = C.logging
# ---
class Current: pass

def UpdateCurrent():
	Current.F = time.time()
	Current.I = int(Current.F)
