# ==
# --Project: Open Grid
# --Author: Sing.Yu <scaner@gmail.com>
# --Module: og.lib.nodeinfo
# ==

import fastmap
from og.lib.utility import MessageReply, MessageError

# -- NIR
class NIR:
	# -- NIR.__init__
	def __init__(self, node):
		self.node = node
		self.msgaddr = {'.': node.nodeid, ':':'nodeinfo'}

		self.basepath = node.config['boot-param']['home']
		self.ni = {}
	
	# -- NIR.set
	def set(self, nodeid, property, value):
		if nodeid not in self.ni:
			self.ni[nodeid] = {}
		self.ni[nodeid][property] = value

	# -- NIR.get
	def get(self, nodeid, property, default = None):
		try:
			return self.ni[nodeid][property]
		except KeyError:
			return default
			
	# -- NIR.del
	def drop(self, nodeid, property = None):
		if property == None:
			try: 
				del self.ni[nodeid]
			except KeyError: 
				pass
		else:
			try:
				del self.ni[nodeid][property]
			except KeyError: 
				pass
			
	# -- NIR.nodes
	def nodes(self):
		return self.ni.keys()

	# -- NIR.nodekeys
	def nodekeys(self, nodeid):
		try:
			return self.ni[nodeid].keys()
		except KeyError:
			return ()

	# -- NIR.loads
	def loads(self, fn = None):
		if not fn:
			fn = self.basepath + '/var/nodeinfo/db.fastmap'

		fin = file(fn, 'r')
		contents = fin.read()
		fin.close()
		R = fastmap.fastloads(contents)
		self.ni = R['nodeinfo']

	# -- NIR.dumps
	def dumps(self, fn = None):
		if not fn:
			fn = self.basepath + '/var/nodeinfo/db.fastmap'

		fin = file(fn, 'w')
		fin.write(fastmap.fastdumps({'nodeinfo': self.ni}))
		fin.close()
	
	# -- NIR.onMessage
	def onMessage(self, msg):
		# ==
		# Message: 'set', 'get', 'delete', 'nodes', 'properties'
		# ==
		try:
			P = msg.get('param')
			if msg['_'] == 'S.set':
				self.set(P['node'], P['property'], P['value'])
				res = 'OK'
			elif msg['_'] == 'S.get':
				res = self.get(P['node'], P['property'], P.get('default'))
			elif msg['_'] == 'S.del':
				self.drop(P['node'], P.get('property'))
				res = 'OK'
			elif msg['_'] == 'S.nodes':
				res = self.nodes()
			elif msg['_'] == 'S.properties':
				res = self.nodekeys(P['node'])
			elif msg['_'] == 'S.load':
				self.loads()
				res = 'OK'
			elif msg['_'] == 'S.save':
				self.dumps()
				res = 'OK'
			else:
				self.node.routing(MessageError(msg,
					self.msgaddr, 'S.NODEINFO', 'UNKONWN-MESSAGE'))
				return
		except Exception, e:
			self.node.routing(MessageError(msg, self.msgaddr,
				'S.NODEINFO', 'Exception:%s' % repr(e)))
			return

		remsg = MessageReply(msg)
		remsg['='] = res
		self.node.routing(remsg)
		return
# -- End of NIR

NodeInfoRepository = NIR

__all__ = ('NodeInfoRepository',)
