# ==
# --Project: OpenGrid
# --Author: Sing.Yu <scaner@gmail.com>
# --UnitTest: og.lib.network
# ==

import os, sys
import unittest

from og.tests.mocksocket import MockSocket
import og.lib.network

from og.lib.network import Pack, Unpack, Unpack0, Unpack_
from og.lib.network import Str2Dword, Dword2Str
from og.lib.network import OGClient

class TestOgLibNetwork(unittest.TestCase):
	def test_SimplePack(self):
		s = Pack(('Call', {'abc': 1, 'def':2}))
		l, v = Unpack(s)

		assert l == len(s) - 4
		assert v[0] == 'Call'
		assert v[1]['abc'] == 1

		v = Unpack0(s)
		assert v[1]['def'] == 2
		
		v = Unpack_(s[4:])
		assert v[0] == 'Call'
	
	
	# ---
	def test_StrAndDword(self):
		s = Dword2Str(300)
		v = Str2Dword(s)
		
		assert v == 300
		assert len(s) == 4
	
	# ---
	def test_OGClient(self):
		C = OGClient(None)
		ms = MockSocket()
		C.socket = ms
		
		ms.outBuffer = Pack({'return': 'OK', 'message':'Hello World'})
		
		r = C.Call({'do':'test'})
		
		assert r['return'] == 'OK'
		assert r['message'] == 'Hello World'
		
		r = Unpack0(ms.inBuffer)
		assert r['do'] == 'test'
		
		
