# ==
# --Project: OpenGrid
# --Author: Sing.Yu <scaner@gmail.com>
# --UnitTest: og.lib.nodeinfo
# ==

import os, sys
import unittest

import og.lib.nodeinfo

class MockNode:
	def __init__(self):
		self.config = {'boot-param':{'home': '/tmp'}}
		self.nodeid = 'testnode'

class TestOgLibNodeinfo(unittest.TestCase):
	def setUp(self):
		self.node = MockNode()
		self.ni = og.lib.nodeinfo.NodeInfoRepository(self.node)

	def test_GetAndSet(self):
		ni = self.ni
		ni.set('dev', 'ip', '10.2.3.4')
		assert ni.get('dev', 'ip') == '10.2.3.4'

		ni.set('dev', 'ip', '10.2.3.3')
		assert ni.get('dev', 'ip') == '10.2.3.3'

		assert ni.get('dev', 'none', 'hello') == 'hello'
		assert ni.get('dx20', 'none', 'world') == 'world'
	
	def test_Drop(self):
		ni = self.ni

		ni.set('dev', 'ip', '10.2.3.4')
		ni.drop('dev')
		assert ni.get('dev', 'ip') == None

		ni.set('dev', 'ip', '10.2.3.4')
		ni.drop('dev', 'ip')
		assert ni.get('dev', 'ip') == None
	
	def test_Keys(self):
		ni = self.ni
		ni.set('dev', 'ip', '10.2.3.4')
		ni.set('dev', 'name', 'dev')

		assert ni.nodekeys('dev') == ['ip', 'name']
