# ==
# --Project: OpenGrid
# --Author: Sing.Yu <scaner@gmail.com>
# --UnitTest: og.lib.tests_poll
# ==

import os, sys
import fcntl, errno
import select
import unittest

def NonBlock(fd):
	flags = fcntl.fcntl(fd, fcntl.F_GETFL, 0)
	flags = flags | os.O_NONBLOCK
	fcntl.fcntl(fd, fcntl.F_SETFL, flags)


class TestOgLibPoll(unittest.TestCase):
	def test_PipeSimple0(self):
		r0, w0 = os.pipe()
		words = 'Hello World'

		os.write(w0, words)
		word = os.read(r0, len(words))
		assert words == word

		os.write(w0, words)
		word = os.read(r0, 200)
		assert words == word

		# os.read(r0, 200) Will be block
		os.close(w0)
		word = os.read(r0, 200)
		assert word == ''
		os.close(r0)
	
		r0, w0 = os.pipe()
		os.close(r0)
		_errno = 0
		try:
			# Write into broken pipe, will raise a OSError with errno = 32
			os.write(w0, words)
		except OSError, why:
			if why[0] != 32: # BROKEN PIPE
				raise
			_errno = why[0]
		assert _errno == 32
		os.close(w0)

	def test_PipeAsync(self):
		words = 'Hello World'
		r0, w0 = os.pipe()
		NonBlock(r0)
		NonBlock(w0)

		_errno = 0
		try:
			word = os.read(r0, 300)
		except OSError, why:
			if why[0] != errno.EWOULDBLOCK:
				raise
			_errno = why[0]

		assert _errno == errno.EWOULDBLOCK
		os.write(w0, words)
		word = os.read(r0, 300)
		assert word == words

		os.close(r0)
		os.close(w0)
	
	def test_PipePollReadClosed(self):
		poll = select.poll()
		r0, w0 = os.pipe()

		NonBlock(r0)
		NonBlock(w0)

		poll.register(w0, select.POLLOUT)

		ev = poll.poll(0)
		assert len(ev) == 1
		assert ev[0] == (w0, select.POLLOUT)

		poll.register(w0, select.POLLIN)
		ev = poll.poll(0)
		assert len(ev) == 0

		os.close(r0)
		ev = poll.poll(0)
		assert len(ev) == 1
		assert ev[0] == (w0, select.POLLERR)

		poll.unregister(w0)
		os.close(w0)
	
	def test_PipePollWriteClosed(self):
		poll = select.poll()
		r0, w0 = os.pipe()

		NonBlock(r0)
		NonBlock(w0)

		poll.register(r0, select.POLLIN)
		os.close(w0)
		ev = poll.poll(0)
		assert len(ev) == 1
		assert ev[0] == (r0, select.POLLHUP)

		word = os.read(r0, 100)
		assert word == ''

		ev = poll.poll(0)
		assert len(ev) == 1
		assert ev[0] == (r0, select.POLLHUP)

		os.close(r0)
