# ==
# --Project: OpenGrid
# --Author: Sing.Yu <scaner@gmail.com>
# --UnitTest: og.lib.process
# ==

import os, sys
import unittest

import tempfile

import og.lib.process
import og.lib.logging

from og.lib.process import PI

# -- helper classes
class MockManager:
	def __init__(self):
		self.procs = {}

# -- Test Suites
class TestOgLibProcess(unittest.TestCase):
	def setUp(self):
		og.lib.logging.logCore.stdout = False
		
	def test_Fork1(self):
		manager = MockManager()

		# get a random filepath
		fd, fn = tempfile.mkstemp('.ut')
		os.close(fd)

		p = PI(manager, args = ['sh', '-c', 'echo -n "Hello World" > %s' % fn])
		pid = p.fire()

		pid2, status = os.wait()
		assert pid2 == pid

		fin = file(fn, 'r')
		text = fin.read()
		fin.close()
		assert text == 'Hello World'
		
		# -- Test with env
		p = PI(manager, args = ['sh', '-c', 'echo -n $OPENGRID_ID > %s' % fn],
			env = {'OPENGRID_ID': 'Hello_World'})
		pid = p.fire()

		pid2, status = os.wait()
		assert pid2 == pid

		fin = file(fn, 'r')
		text = fin.read()
		fin.close()
		assert text == 'Hello_World'

