import os, sys
import unittest

import og.lib.utility

class TestOgLibUtility(unittest.TestCase):
	def test_LoadConfig0(self):
		import tempfile
		fd, fname = tempfile.mkstemp('.ut')
		os.write(fd, '''
_CONF = {
	'unit_test': 'READY'
}
''')
		os.close(fd)
		C = og.lib.utility.LoadConfig0(fname)
		assert C['unit_test'] == 'READY'

	# --
	def test_LoadConfig3(self):
		import tempfile
		fd, fname = tempfile.mkstemp('.ut')
		os.write(fd, '''
_CONF = {
	'unit_test': g['ut'],
	'unit_test_2': l['ut'],
}
''')
		os.close(fd)
		C = og.lib.utility.LoadConfig3(fname, {'g':{'ut':'Global'}}, {'l':{'ut':'Local'}})
		assert C['unit_test'] == 'Global'
		assert C['unit_test_2'] == 'Local'


	def test_ListenAddr(self):
		ListenAddr = og.lib.utility.ListenAddr

		assert ListenAddr(None, 3000) == ('0.0.0.0', 3000)
		assert ListenAddr(2999, None) == ('0.0.0.0', 2999)
		assert ListenAddr(('127.0.0.1', 3000), None) == ('127.0.0.1', 3000)
	
	def test_MessageReply(self):
		MR = og.lib.utility.MessageReply

		v = MR({
			'#': 333,
			'_': 'action',
		})

		assert v['#'] == 334
		assert v['_'] == 'action'
		assert v.get('~') == None

		v = MR({
			'@': 'source-0',
			'!': 'dest-0',
			'_': 'action2',
		})

		assert v['@'] == 'dest-0'
		assert v['!'] == 'source-0'
	
	def test_SerialGenrator(self):
		assert og.lib.utility.Serial() == 2
		assert og.lib.utility.Serial() == 4
		assert og.lib.utility.Serial() == 6
		assert og.lib.utility.Serial() == 8


