# ==
# --Project: Open Grid
# --Author: Sing Yu <scaner@gmail.com>
# --Module: og.lib.utility
# ==

import sys, os
import random
import og.constants

__all__ = [
	'LoadConfig0', 'LoadConfig3', 
	'ListenAddr', 'CompactTrackback',
	'FirstExists', 'SetUidAndGid',
	'MessageReply', 'MessageError',
	'RandomString', 'Serial',
	# --
	'SecondTimer',
]


# -- LoadConfig0
def LoadConfig0(fname):
	fin = file(fname, 'r')
	codes = fin.read()
	fin.close()
	exec codes
	return _CONF

def LoadConfig3(fname, g, l):
	if g is None:
		g = {}
	if l is None:
		l = {}

	fin = file(fname, 'r')
	codes = fin.read()
	fin.close()
	exec codes in g, l
	return l['_CONF']

# -- ListenAddr
def ListenAddr(v, default):
	if not v:
		v = default
	if type(v) == int:
		return ('0.0.0.0', v)
	else:
		return v

# -- CompactTrackback
def CompactTrackback():
	t, v, tb = sys.exc_info()
	tbinfo = []
	
	assert tb
	
	while tb:
		tbinfo.append((
			tb.tb_frame.f_code.co_filename,
			tb.tb_frame.f_code.co_name,
			str(tb.tb_lineno)))
		tb = tb.tb_next
	del tb
	
	file, function, line = tbinfo[-1]
	info = '\n'.join(['[%s|%s|%s]' % x for x in tbinfo])
	return (file, function, line), t, v, info

# -- FirstExists
def FirstExists(*fns):
	for fn in fns:
		if os.path.isfile(fn) and os.access(fn, os.R_OK):
			return fn
	return None


# == == == == == == == == == == == == == == == == == == == == == == #
# -- SecondTimer
class SecondTimer:
	# -- SecondTimer.__init__
	def __init__(self, handler):
		self.handler = handler
		self.callbacks = []
	
	# -- SecondTimer.addCallback
	def addCallback(self, cbname, cb, interval):
		self.callbacks.append([cbname, cb, interval, 0])
	
	# -- SecondTimer.touch
	def touch(self, cur):
		for cbi in self.callbacks:
			if cur - cbi[3] > cbi[2]:
				cbi[3] = cur
				cbi[1](self.handler, cur)
	
	# -- SecondTimer.resetAll
	def resetAll(self, cur):
		for cbi in self.callbacks:
			cbi[3] = cur
	
	# -- SecondTimer.triggerAll:
	def triggerAll(self, cur):
		for cbi in self.callbacks:
			cbi[3] = cur
			cbi[1](self.handler, cur)

	# -- SecondTimer.setInterval
	def setInterval(self, cbname, interval):
		for cbi in self.callbacks:
			if cbi[0] == cbname:
				cbi[2] = interval
				return True

		return False

# == == == == == == == == == == == == == == == == == == == == == == #
def SetUidAndGid():
	uid = os.getuid()
	gid = os.getgid()

	if gid == 0 and gid != og.constants.OPENGRID_GID:
		os.setgid(og.constants.OPENGRID_GID)
	if uid == 0 and uid != og.constants.OPENGRID_UID:
		os.setuid(og.constants.OPENGRID_UID)

# == == == == == == == == == == == == == == == == == == == == == == #
def MessageReply(msg):
	remsg = {}
	dest = msg.get('!')
	src = msg.get('@')
	action = msg.get('_')
	serial = msg.get('#', 0)
	tag = msg.get('~')

	if src: remsg['!'] = src
	if dest: remsg['@'] = dest
	if action: remsg['_'] = action
	if tag: remsg['~'] = tag
	remsg['#'] = serial + 1

	return remsg

# --
def MessageError(msg, sendid, why, where):
	remsg = MessageReply(msg)
	remsg['#'] -= 1
	remsg['@'] = sendid
	remsg['$'] = (why, where)
	return remsg

# --
def RandomString(length, chars = None):
	if not chars:
		chars = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
	
	nchars = len(chars) - 1
	return ''.join([chars[random.randint(0, nchars)] for i in range(length)])

# --
class SerialGenerator(object):
	def __init__(self):
		self.serial = 0
	
	def __call__(self):
		self.serial += 2
		return self.serial
	
	def current(self):
		return self.serial
	
Serial = SerialGenerator()

